import 'dart:convert';

import 'exceptions.dart';

class Musician {
  final MusicianType type;
  final String firstName;
  final String lastName;
  final Date birth;
  final Date death;
  final String label;

  String get name {
    if (type == MusicianType.other) {
      if (label != null && label.isNotEmpty) {
        return label;
      } else {
        return 'Unbekannt';
      }
    } else {
      if (firstName != null && firstName.isNotEmpty) {
        if (lastName != null && lastName.isNotEmpty) {
          return '$firstName $lastName';
        } else {
          return firstName;
        }
      } else if (lastName != null && lastName.isNotEmpty) {
        return lastName;
      } else {
        return 'Unbekannt';
      }
    }
  }

  const Musician({
    this.type,
    this.firstName,
    this.lastName,
    this.birth,
    this.death,
    this.label,
  });

  factory Musician.fromString(String source) {
    try {
      return Musician.fromJson(jsonDecode(source));
    } on FormatException {
      throw JsonSourceException('FormatException reading musician', source);
    } on TypeError {
      throw JsonSourceException('TypeError reading musician', source);
    }
  }

  factory Musician.fromJson(Map<String, dynamic> json) {
    try {
      final int typeIndex = json['type'];
      if (typeIndex == null) {
        throw JsonInputException('Musician without type', json);
      }

      final Map<String, dynamic> birthJson = json['birth'];
      final Map<String, dynamic> deathJson = json['death'];

      return Musician(
        type: MusicianType.values[typeIndex],
        firstName: json['firstName'],
        lastName: json['lastName'],
        birth: birthJson != null ? Date.fromJson(json['date']) : null,
        death: deathJson != null ? Date.fromJson(json['death']) : null,
        label: json['label'],
      );
    } on RangeError {
      throw JsonInputException('Unknown musician type', json);
    } on TypeError {
      throw JsonInputException('TypeError reading musician', json);
    }
  }

  String toString() => jsonEncode(toJson());

  Map<String, dynamic> toJson() => {
    'type': type.index,
    'firstName': firstName,
    'lastName': lastName,
    'birth': birth,
    'death': death,
    'label': label,
  };
}

enum MusicianType {
  person,
  other,
}

class Date {
  final int day;
  final int month;
  final int year;

  const Date({
    this.day,
    this.month,
    this.year,
  });

  factory Date.fromJson(Map<String, dynamic> json) {
    try {
      return Date(
        day: json['day'],
        month: json['month'],
        year: json['year'],
      );
    } on TypeError {
      throw JsonInputException('TypeError reading date', json);
    }
  }

  Map<String, dynamic> toJson() => {
    'day': day,
    'month': month,
    'year': year,
  };
}
