import 'dart:convert';

import 'exceptions.dart';

class Work {
  final String name;

  final int era;
  final int composer;
  final int year;
  
  final int genre;
  final int number;
  final int key;
  
  final int relatesTo;
  final int partOf;
  final int index;
  

  const Work({
    this.name,
    this.era,
    this.composer,
    this.year,
    this.genre,
    this.number,
    this.key,
    this.relatesTo,
    this.partOf,
    this.index,
  });

  factory Work.fromString(String source) {
    try {
      return Work.fromJson(jsonDecode(source));
    } on FormatException {
      throw JsonSourceException('FormatException reading work', source);
    } on TypeError {
      throw JsonSourceException('TypeError reading work', source);
    }
  }

  factory Work.fromJson(Map<String, dynamic> json) {
    try {
      return Work(
        name: json['name'],
        era: json['era'],
        composer: json['composer'],
        year: json['year'],
        genre: json['genre'],
        number: json['number'],
        key: json['key'],
        relatesTo: json['relatesTo'],
        partOf: json['partOf'],
        index: json['index'],
      );
    } on TypeError {
      throw JsonInputException('TypeError reading work', json);
    }
  }

  String toString() => jsonEncode(toJson());

  Map<String, dynamic> toJson() => {
    'name': name,
    'era': era,
    'composer': composer,
    'year': year,
    'genre': genre,
    'number': number,
    'key': key,
    'relatesTo': relatesTo,
    'partOf': partOf,
    'index': index,
  };
}
